/*
 * Daneel - Dalvik to Java bytecode compiler
 * Copyright (C) 2011  IcedRobot team
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * This file is subject to the "Classpath" exception:
 *
 * Linking this library statically or dynamically with other modules is
 * making a combined work based on this library.  Thus, the terms and
 * conditions of the GNU General Public License cover the whole
 * combination.
 *
 * As a special exception, the copyright holders of this library give you
 * permission to link this library with independent modules to produce an
 * executable, regardless of the license terms of these independent
 * modules, and to copy and distribute the resulting executable under terms
 * of your choice, provided that you also meet, for each linked independent
 * module, the terms and conditions of the license of that module.  An
 * independent module is a module which is not derived from or based on
 * this library.  If you modify this library, you may extend this exception
 * to your version of the library, but you are not obligated to do so.  If
 * you do not wish to do so, delete this exception statement from your
 * version.
 */

package org.icedrobot.daneel.rewriter;

import static org.junit.Assert.assertEquals;

import org.icedrobot.daneel.DexifyingRunner;
import org.junit.Test;
import org.junit.runner.RunWith;

/**
 * This is a regression test for a bug which happens if you actually stick to
 * the Dalvik VM bytecode specification. The long versions of the shifting
 * operations actually have a different signature:
 * <pre>
 *   shr-long
 *   int64 a, int32 b;
 *   int64 result = a >> (b & 0x3f);
 * </pre>
 */
@RunWith(DexifyingRunner.class)
public class LongShiftTest {

    @Test
    public void testShl() {
        assertEquals(23 << 1, DEXCode.shl(23, 1));
        assertEquals(42 << 2, DEXCode.shl(42, 2));
        assertEquals(-1 << 18, DEXCode.shl(-1, 18));
    }

    @Test
    public void testShr() {
        assertEquals(23 >> 1, DEXCode.shr(23, 1));
        assertEquals(42 >> 2, DEXCode.shr(42, 2));
        assertEquals(-1 >> 36, DEXCode.shr(-1, 36));
    }

    @Test
    public void testUshr() {
        assertEquals(23 >>> 1, DEXCode.ushr(23, 1));
        assertEquals(42 >>> 2, DEXCode.ushr(42, 2));
        assertEquals(-1 >>> 36, DEXCode.ushr(-1, 36));
    }

    // Keep this class named "DEXCode" to push it through Daneel.
    private static class DEXCode {
        public static int shl(long val, int shift) {
            return (int) (val << shift);
        }

        public static int shr(long val, int shift) {
            return (int) (val >> shift);
        }

        public static int ushr(long val, int shift) {
            return (int) (val >>> shift);
        }
    };
}
